from fastapi import APIRouter, Depends, HTTPException, Query
from sqlalchemy.orm import Session
from typing import Any, Optional

from app import crud
from app.api import deps

from app.schemas.lesson import (
    Lesson, 
    LessonCreate,
    LessonSearchResults,
)

from app.models.user import User

router = APIRouter()


@router.get(
    "/{lesson_id}", 
    status_code=200,
    response_model=Lesson)
def fetch_lesson(
    *, 
    lesson_id: int,
    db: Session = Depends(deps.get_db),
    current_user: User = Depends(deps.get_current_user),
    ) -> dict:
    """
    Fetch a single lesson by ID
    """

    result = crud.lesson.get(db=db, id=lesson_id)
    
    if not result:
        raise HTTPException(
            status_code=404, 
            detail=f"Lesson with ID {lesson_id} not found"
        )
    
    return result


@router.post(
    "/", 
    status_code=201, 
    response_model=Lesson)
def create_lesson(
    *, 
    lesson_in: LessonCreate, 
    db: Session = Depends(deps.get_db),
    current_user: User = Depends(deps.get_current_user),
    ) -> dict:
    """
    Create a new lesson
    """
    
    lesson = crud.lesson.create(db=db, obj_in=lesson_in)

    return lesson


@router.get(
    "/", 
    status_code=200,
    response_model=LessonSearchResults)
def get_all_lessons(
    max_results: Optional[int] = 10,
    db: Session = Depends(deps.get_db),
    current_user: User = Depends(deps.get_current_user),
    ) -> dict:
    """
    Get all lessons
    """
    
    return {
        "results": list(
            crud.lesson.get_multi(db=db, limit=max_results))}
