import React, { useEffect, useState } from "react";
import FastAPIClient from "../../client";
import config from "../../config";
import DashboardHeader from "../../components/DashboardHeader";
import Footer from "../../components/Footer";
import jwtDecode from "jwt-decode";
import * as moment from "moment";
import LessonTable from "../../components/LessonTable";
import FormInput from "../../components/FormInput/FormInput";
import Button from "../../components/Button/Button";
import { NotLoggedIn } from "./NotLoggedIn";
import Loader from "../../components/Loader";
import PopupModal from "../../components/Modal/PopupModal";

const client = new FastAPIClient(config);

const ProfileView = ({ lessons }) => {
	return (
		<>
			<LessonTable
				lessons={lessons}
				
				showUpdate={true}
			/>
			
		</>
	);
};

const LessonDashboard = () => {
	const [isLoggedIn, setIsLoggedIn] = useState(false);
	const [error, setError] = useState({ title: "", content: "", tags: "" });
	const [lessonForm, setLessonForm] = useState({
		title: "",
		content: "",
		tags: "",
	});

	const [showForm, setShowForm] = useState(false);
	const [lessons, setLessons] = useState([]);

	const [loading, setLoading] = useState(false);
	const [refreshing, setRefreshing] = useState(true);

	useEffect(() => {
		fetchUserLessons();
	}, []);

	const fetchUserLessons = () => {
		client.getUserLessons().then((data) => {
			setRefreshing(false);
			setLessons(data?.results);
		});
	};

   const contentPatternValidation = () => {
         return true;
        };

	const onCreateLesson = (e) => {
		e.preventDefault();
		setLoading(true);
		setError(false);

		if (lessonForm.title.length <= 0) {
			setLoading(false);
			return setError({ title: "Please Enter Lesson Title" });
		}
		if (lessonForm.content.length <= 0) {
			setLoading(false);
			return setError({ content: "Please Enter Lesson Content" });
		}
		if (!contentPatternValidation(lessonForm.content)) {
			setLoading(false);
			return setError({ content: "Please Enter Valid Content" });
		}
		if (lessonForm.tags.length <= 0) {
			setLoading(false);
			return setError({ tags: "Please Enter Lesson Tags" });
		}

		client.fetchUser().then((user) => {
			client
				.createLesson(
					lessonForm.title,
					lessonForm.content,
					lessonForm.tags,
					user?.id
				)
				// eslint-disable-next-line no-unused-vars
				.then((data) => {  // eslint:ignore
					fetchUserLessons();
					setLoading(false);
					setShowForm(false);
				});
		});
	};

	useEffect(() => {
		const tokenString = localStorage.getItem("token");
		if (tokenString) {
			const token = JSON.parse(tokenString);
			const decodedAccessToken = jwtDecode(token.access_token);
			if (moment.unix(decodedAccessToken.exp).toDate() > new Date()) {
				setIsLoggedIn(true);
			}
		}
	}, []);

	if (refreshing) return !isLoggedIn ? <NotLoggedIn /> : <Loader />;

	return (
		<>
			<section
				className="flex flex-col bg-black text-center"
				style={{ minHeight: "100vh" }}
			>
				<DashboardHeader />
				<div className="container px-5 pt-6 text-center mx-auto lg:px-20">
						{/*TODO - move to component*/}
					<h1 className="mb-12 text-3xl font-medium text-white">
						Lessons Learned
					</h1>

					<button
						className="my-5 text-white bg-teal-500 p-3 rounded"
						onClick={() => {
							setShowForm(!showForm);
						}}
					>
						Create Lesson
					</button>

					<p className="text-base leading-relaxed text-white">Latest lessons</p>
					<div className="mainViewport text-white">
						{lessons.length && (
							<ProfileView
								lessons={lessons}
								fetchUserLessons={fetchUserLessons}
							/>
						)}
					</div>
				</div>

				<Footer />
			</section>
			{showForm && (
				<PopupModal
					modalTitle={"Create Lesson"}
					onCloseBtnPress={() => {
						setShowForm(false);
						setError({ fullName: "", email: "", password: "" });
					}}
				>
					<div className="mt-4 text-left">
						<form className="mt-5" onSubmit={(e) => onCreateLesson(e)}>
							<FormInput
								type={"text"}
								name={"title"}
								label={"Title"}
								error={error.title}
								value={lessonForm.title}
								onChange={(e) =>
									setLessonForm({ ...lessonForm, title: e.target.value })
								}
							/>
							<FormInput
								type={"text"}
								name={"content"}
								label={"Content"}
								error={error.content}
								value={lessonForm.content}
								onChange={(e) =>
									setLessonForm({ ...lessonForm, content: e.target.value })
								}
							/>
							<FormInput
								type={"text"}
								name={"tags"}
								label={"Tags"}
								error={error.tags}
								value={lessonForm.tags}
								onChange={(e) =>
									setLessonForm({ ...lessonForm, tags: e.target.value })
								}
							/>
							<Button
								loading={loading}
								error={error.tags}
								title={"Create Lesson"}
							/>
						</form>
					</div>
				</PopupModal>
			)}
		</>
	);
};

export default LessonDashboard;
